@@ -51,7 +51,8 @@ namespace Mono.Debugger.Backend
 		CreateAppDomain,
 		UnloadAppDomain,
 
-		Trampoline	= 256
+		OldTrampoline	= 256,
+		Trampoline	= 512
 	}
 
 	internal delegate bool ManagedCallbackFunction (SingleSteppingEngine engine);
@@ -292,8 +293,16 @@ namespace Mono.Debugger.Backend
 				}
 
 				case NotificationType.GcThreadExited:
+					Report.Debug (DebugFlags.Threads, "{0} gc thread exited", engine);
 					engine.OnManagedThreadExited ();
-					break;
+					try {
+						inferior.Continue ();
+					} catch {
+						// Ignore errors; for some reason, the thread may have died
+						// already by the time get this notification.
+					}
+					resume_target = false;
+					return true;
 
 				case NotificationType.InitializeThreadManager:
 					csharp_language = inferior.Process.CreateMonoLanguage (
@@ -313,12 +322,14 @@ namespace Mono.Debugger.Backend
 					Inferior.StackFrame iframe = inferior.GetCurrentFrame (false);
 					engine.SetMainReturnAddress (iframe.StackPointer);
 					engine.ProcessServant.OnProcessReachedMainEvent ();
-					resume_target = !engine.OnModuleLoaded (null);
+					resume_target = !engine.InitializeBreakpoints ();
 					return true;
 				}
 
 				case NotificationType.WrapperMain:
+					break;
 				case NotificationType.MainExited:
+					engine.SetMainReturnAddress (TargetAddress.Null);
 					break;
 
 				case NotificationType.UnhandledException:
@@ -348,6 +359,7 @@ namespace Mono.Debugger.Backend
 					csharp_language = null;
 					break;
 
+				case NotificationType.OldTrampoline:
 				case NotificationType.Trampoline:
 					resume_target = false;
 					return false;
@@ -357,9 +369,9 @@ namespace Mono.Debugger.Backend
 
 				case NotificationType.InterruptionRequest:
 					inferior.WriteInteger (MonoDebuggerInfo.InterruptionRequest, 0);
-					engine.OnManagedCallback (managed_callbacks);
+					var callbacks = managed_callbacks;
 					managed_callbacks = new Queue<ManagedCallbackData> ();
-					resume_target = false;
+					resume_target = !engine.OnManagedCallback (callbacks);
 					return true;
 
 				default: {
@@ -476,6 +488,20 @@ namespace Mono.Debugger.Backend
 			return new MonoDebuggerInfo (memory, reader);
 		}
 
+		public bool CheckRuntimeVersion (int major, int minor)
+		{
+			if (MajorVersion < major)
+				return false;
+			if (MajorVersion > major)
+				return true;
+			return MinorVersion >= minor;
+		}
+
+		public bool HasNewTrampolineNotification ()
+		{
+			return CheckRuntimeVersion (80, 2) || CheckRuntimeVersion (81, 4);
+		}
+
 		protected MonoDebuggerInfo (TargetMemoryAccess memory, TargetReader reader)
 		{
 			reader.Offset = 8;
