@@ -99,6 +99,10 @@ namespace Mono.Debugger.Backend
 			get { return is_attached; }
 		}
 
+		public bool CanDetach {
+			get { return is_attached; }
+		}
+
 		public bool IsExeced {
 			get { return is_execed; }
 		}
@@ -504,7 +508,7 @@ namespace Mono.Debugger.Backend
 
 		public void Detach ()
 		{
-			if (!IsAttached)
+			if (!CanDetach)
 				throw new TargetException (TargetError.CannotDetach);
 
 			main_thread.Detach ();
@@ -664,9 +668,9 @@ namespace Mono.Debugger.Backend
 				      "Released global thread lock #1: {0}", caller);
 		}
 
-		public void ActivatePendingBreakpoints (CommandResult result)
+		public bool ActivatePendingBreakpoints (CommandResult result)
 		{
-			((SingleSteppingEngine) main_thread).ManagedCallback (
+			return ((SingleSteppingEngine) main_thread).ManagedCallback (
 				delegate (SingleSteppingEngine sse) {
 					return sse.ActivatePendingBreakpoints (null);
 				}, result);
