@@ -171,7 +171,6 @@ server_ptrace_detach (ServerHandle *handle)
 static ServerCommandError
 server_ptrace_kill (ServerHandle *handle)
 {
-	kill (handle->inferior->pid, SIGKILL);
 	if (ptrace (PTRACE_KILL, handle->inferior->pid, NULL, 0))
 		return COMMAND_ERROR_UNKNOWN_ERROR;
 
@@ -347,6 +346,9 @@ server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg
 			*data1 = retval;
 			*data2 = retval2;
 			return MESSAGE_RUNTIME_INVOKE_DONE;
+
+		case STOP_ACTION_INTERNAL_ERROR:
+			return MESSAGE_INTERNAL_ERROR;
 		}
 
 		g_assert_not_reached ();
