@@ -11,12 +11,14 @@ namespace Mono.Debugger
 	[Serializable]
 	public sealed class ExceptionCatchPoint : Event
 	{
-		int handle = -1;
-
 		public override bool IsPersistent {
 			get { return true; }
 		}
 
+		public override bool NeedsActivation {
+			get { return false; }
+		}
+
 		public bool Unhandled {
 			get; set;
 		}
@@ -35,55 +37,22 @@ namespace Mono.Debugger
 		}
 
 		public override bool IsActivated {
-			get { return handle > 0; }
+			get { return true; }
 		}
 
 		public override void Activate (Thread target)
-		{
-			lock (this) {
-				EnableCatchpoint (target);
-			}
-		}
+		{ }
 
 		public override void Deactivate (Thread target)
-		{
-			lock (this) {
-				DisableCatchpoint (target);
-			}
-		}
+		{ }
 
 		internal override void OnTargetExited ()
 		{
 			exception = null;
-			handle = -1;
 		}
 
 		public override void Remove (Thread target)
-		{
-			lock (this) {
-				DisableCatchpoint (target);
-			}
-		}
-
-		void EnableCatchpoint (Thread target)
-		{
-			lock (this) {
-				if (handle > 0)
-					return;
-
-				handle = target.AddEventHandler (this);
-			}
-		}
-
-		void DisableCatchpoint (Thread target)
-		{
-			lock (this) {
-				if (handle > 0)
-					target.RemoveEventHandler (handle);
-
-				handle = -1;
-			}
-		}
+		{ }
 
 		bool IsSubclassOf (TargetMemoryAccess target, TargetStructType type,
 				   TargetType parent)
