@@ -8,277 +8,9 @@ using ST=System.Threading;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
 using Mono.Debugger.Languages.Mono;
-using EE=Mono.Debugger.ExpressionEvaluator;
 
 namespace Mono.Debugger.Frontend
 {
-	public class ExpressionParser : IExpressionParser
-	{
-		public readonly Interpreter Interpreter;
-
-		private readonly CSharp.ExpressionParser parser;
-
-		internal ExpressionParser (Interpreter interpreter)
-		{
-			this.Interpreter = interpreter;
-
-			parser = new CSharp.ExpressionParser ("C#");
-		}
-
-		/*
-		 * This version throws an `ExpressionParsingException' containing a detailed
-		 * error message and the location of the error.
-		 *
-		 * Use ScriptingContext.ParseExpression() to get a `ScriptingException' with
-		 * a user-readable error message.
-		 */
-		internal Expression ParseInternal (string text)
-		{
-			return parser.Parse (text);
-		}
-
-		public EE.IExpression Parse (string text)
-		{
-			return new MyExpression (this, parser.Parse (text));
-		}
-
-		protected static SourceLocation FindFile (ScriptingContext context, string filename,
-							  int line)
-		{
-			SourceFile file = context.Interpreter.Session.FindFile (filename);
-			if (file == null)
-				throw new ScriptingException ("Cannot find source file `{0}'.",
-							      filename);
-
-			MethodSource source = file.FindMethod (line);
-			if (source == null)
-				throw new ScriptingException (
-					"Cannot find method corresponding to line {0} in `{1}'.",
-					line, file.Name);
-
-			return new SourceLocation (source, file, line);
-		}
-
-		protected SourceLocation DoParseExpression (ScriptingContext context,
-							    LocationType type, string arg)
-		{
-			Expression expr = context.ParseExpression (arg);
-			MethodExpression mexpr = expr.ResolveMethod (context, type);
-
-			if (mexpr != null)
-				return mexpr.EvaluateSource (context);
-			else
-				return context.FindMethod (arg);
-		}
-
-		public static bool ParseLocation (ScriptingContext context, string arg,
-						  out SourceLocation location)
-		{
-			int line;
-			int pos = arg.IndexOf (':');
-			if (pos >= 0) {
-				string filename = arg.Substring (0, pos);
-				try {
-					line = (int) UInt32.Parse (arg.Substring (pos+1));
-				} catch {
-					throw new ScriptingException ("Expected filename:line");
-				}
-
-				location = FindFile (context, filename, line);
-				return true;
-			}
-
-			try {
-				line = (int) UInt32.Parse (arg);
-			} catch {
-				location = null;
-				return false;
-			}
-
-			StackFrame frame = context.CurrentFrame;
-			if ((frame == null) || (frame.SourceLocation == null) ||
-			    (frame.SourceLocation.FileName == null))
-				throw new ScriptingException (
-					"Current stack frame doesn't have source code");
-
-			location = FindFile (context, frame.SourceLocation.FileName, line);
-			return true;
-		}
-
-		protected SourceLocation DoParse (ScriptingContext context, LocationType type,
-						  string arg)
-		{
-			if (type != LocationType.Default)
-				return DoParseExpression (context, type, arg);
-
-			SourceLocation location;
-			if (ParseLocation (context, arg, out location))
-				return location;
-
-			return DoParseExpression (context, type, arg);
-		}
-
-		public SourceLocation ParseLocation (Thread target, StackFrame frame,
-						     LocationType type, string arg)
-		{
-			ScriptingContext context = new ScriptingContext (Interpreter);
-			context.CurrentThread = target;
-			context.CurrentFrame = frame;
-
-			try {
-				return DoParse (context, type, arg);
-			} catch (ScriptingException ex) {
-				throw new TargetException (TargetError.LocationInvalid, ex.Message);
-			}
-		}
-
-		public string EvaluateExpression (ScriptingContext context, string text,
-						  DisplayFormat format)
-		{
-			Expression expression = context.ParseExpression (text);
-
-			try {
-				expression = expression.Resolve (context);
-			} catch (ScriptingException ex) {
-				throw new ScriptingException ("Cannot resolve expression `{0}': {1}",
-							      text, ex.Message);
-			} catch {
-				throw new ScriptingException ("Cannot resolve expression `{0}'.", text);
-			}
-
-			try {
-				object retval = expression.Evaluate (context);
-				return context.FormatObject (retval, format);
-			} catch (ScriptingException ex) {
-				throw new ScriptingException ("Cannot evaluate expression `{0}': {1}",
-							      text, ex.Message);
-			} catch {
-				throw new ScriptingException ("Cannot evaluate expression `{0}'.", text);
-			}
-		}
-
-		protected MyAsyncResult EvaluateAsync (StackFrame frame, MyExpression expression,
-						       EE.EvaluationFlags flags, EE.EvaluationCallback callback)
-		{
-			MyAsyncResult async = new MyAsyncResult (expression);
-
-			ST.ThreadPool.QueueUserWorkItem (delegate {
-				ScriptingContext context = new ScriptingContext (Interpreter);
-				context.InterruptionHandler = async;
-				context.CurrentFrame = frame;
-
-				if ((flags & EE.EvaluationFlags.NestedBreakStates) != 0)
-					context.ScriptingFlags |= ScriptingFlags.NestedBreakStates;
-
-				object data;
-				EE.EvaluationResult result = DoEvaluate (context, expression.Expression, out data);
-				callback (result, data);
-				async.WaitHandle.Set ();
-			});
-
-			return async;
-		}
-
-		EE.EvaluationResult DoEvaluate (ScriptingContext context, Expression expression, out object result)
-		{
-			Expression resolved = null;
-
-			try {
-				resolved = expression.Resolve (context);
-				if (resolved == null) {
-					result = String.Format ("Cannot resolve expression `{0}'", expression.Name);
-					return EE.EvaluationResult.InvalidExpression;
-				}
-
-				result = resolved.Evaluate (context);
-				return EE.EvaluationResult.Ok;
-			} catch (InvocationException ex) {
-				result = ex.Exception;
-				return EE.EvaluationResult.Exception;
-			} catch (ScriptingException ex) {
-				result = ex.Message;
-				return EE.EvaluationResult.InvalidExpression;
-			} catch (EvaluationTimeoutException) {
-				result = null;
-				return EE.EvaluationResult.Timeout;
-			} catch (Exception ex) {
-				result = String.Format ("Cannot resolve expression `{0}': {1}", expression.Name, ex);
-				return EE.EvaluationResult.InvalidExpression;
-			}
-		}
-
-		protected class MyExpression : EE.IExpression
-		{
-			public readonly ExpressionParser Parser;
-			public readonly Expression Expression;
-
-			public string Name {
-				get { return Expression.Name; }
-			}
-
-			public MyExpression (ExpressionParser parser, Expression expr)
-			{
-				this.Parser = parser;
-				this.Expression = expr;
-			}
-
-			public EE.AsyncResult EvaluateAsync (StackFrame frame, EE.EvaluationFlags flags,
-							     EE.EvaluationCallback callback)
-			{
-				return Parser.EvaluateAsync (frame, this, flags, callback);
-			}
-
-			public override string ToString ()
-			{
-				return Expression.Name;
-			}
-		}
-
-		protected class MyAsyncResult : EE.AsyncResult, IInterruptionHandler
-		{
-			public readonly MyExpression Expression;
-			public readonly ST.ManualResetEvent WaitHandle;
-			public readonly ST.ManualResetEvent AbortHandle;
-
-			public MyAsyncResult (MyExpression expr)
-			{
-				this.Expression = expr;
-				this.WaitHandle = new ST.ManualResetEvent (false);
-				this.AbortHandle = new ST.ManualResetEvent (false);
-			}
-
-			public override object AsyncState {
-				get { return Expression; }
-			}
-
-			public override ST.WaitHandle AsyncWaitHandle {
-				get { return WaitHandle; }
-			}
-
-			public override bool CompletedSynchronously {
-				get { return false; }
-			}
-
-			public override bool IsCompleted {
-				get { return WaitHandle.WaitOne (0); }
-			}
-
-			public override void Abort ()
-			{
-				AbortHandle.Set ();
-			}
-
-			ST.WaitHandle IInterruptionHandler.InterruptionEvent {
-				get { return AbortHandle; }
-			}
-
-			bool IInterruptionHandler.CheckInterruption ()
-			{
-				return AbortHandle.WaitOne (0);
-			}
-		}
-	}
-
 	public abstract class Expression
 	{
 		public abstract string Name {
@@ -1828,9 +1560,9 @@ namespace Mono.Debugger.Frontend
 		protected readonly TargetStructObject instance;
 		TargetClass class_info;
 
-		protected StructAccessExpression (TargetStructType type,
-						  TargetStructObject instance,
-						  TargetMemberInfo member)
+		public StructAccessExpression (TargetStructType type,
+					       TargetStructObject instance,
+					       TargetMemberInfo member)
 		{
 			this.Type = type;
 			this.Member = member;
@@ -1839,7 +1571,7 @@ namespace Mono.Debugger.Frontend
 		}
 
 		public override string Name {
-			get { return Member.Name; }
+			get { return Type.Name + "." + Member.Name; }
 		}
 
 		public override TargetStructObject InstanceObject {
