@@ -1,5 +1,6 @@
 using System;
 using System.Text;
+using System.Diagnostics;
 using System.Collections;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
@@ -50,7 +51,6 @@ namespace Mono.Debugger.Frontend
 
 		public void Format (Thread target, TargetObject obj)
 		{
-			sb.Append (String.Format ("({0}) ", obj.TypeName));
 			FormatObjectRecursed (target, obj, false);
 		}
 
@@ -60,6 +60,11 @@ namespace Mono.Debugger.Frontend
 
 			Append ("{0} = ", variable.Name);
 
+			if (!variable.IsAlive (frame.TargetAddress)) {
+				Append ("<optimized out>");
+				return;
+			}
+
 			try {
 				obj = variable.GetObject (frame);
 				if (obj != null)
@@ -112,6 +117,18 @@ namespace Mono.Debugger.Frontend
 			pos = 0;
 		}
 
+		protected void FormatNullable (Thread target, TargetNullableObject nullable)
+		{
+			bool has_value = nullable.HasValue (target);
+
+			if (has_value) {
+				TargetObject value = nullable.GetValue (target);
+				FormatObjectRecursed (target, value, true);
+			} else {
+				Append ("null");
+			}
+		}
+
 		protected void FormatObjectRecursed (Thread target, TargetObject obj, bool recursed)
 		{
 			try {
@@ -140,6 +157,10 @@ namespace Mono.Debugger.Frontend
 					Format (target, value);
 					break;
 
+				case TargetObjectKind.Nullable:
+					FormatNullable (target, (TargetNullableObject) obj);
+					break;
+
 				default:
 					PrintObject (target, obj);
 					break;
@@ -167,7 +188,7 @@ namespace Mono.Debugger.Frontend
 					TargetObject deref = pobj.GetDereferencedObject (target);
 					Append ("&({0}) ", deref.TypeName);
 					FormatObjectRecursed (target, deref, true);
-				} catch {
+				} catch (Exception ex) {
 					FormatObjectRecursed (target, pobj, true);
 				}
 				break;
@@ -202,6 +223,10 @@ namespace Mono.Debugger.Frontend
 				FormatStructObject (target, (TargetStructObject) obj);
 				break;
 
+			case TargetObjectKind.Nullable:
+				FormatNullable (target, (TargetNullableObject) obj);
+				break;
+
 			default:
 				PrintObject (target, obj);
 				break;
@@ -237,6 +262,10 @@ namespace Mono.Debugger.Frontend
 			for (int i = 0; i < fields.Length; i++) {
 				if (fields [i].IsStatic || fields [i].HasConstValue)
 					continue;
+				if (fields [i].IsCompilerGenerated)
+					continue;
+				if (fields [i].DebuggerBrowsableState == DebuggerBrowsableState.Never)
+					continue;
 
 				if (!first) {
 					Append (", ");
