@@ -116,12 +116,19 @@ namespace Mono.Debugger.Languages.Mono
 			get { return !GenericClass.IsNull; }
 		}
 
+		internal int GetInstanceSize (TargetMemoryAccess target)
+		{
+			return MonoRuntime.MonoClassGetInstanceSize (target, KlassAddress);
+		}
+
 		internal MonoFieldInfo[] GetFields (TargetMemoryAccess target)
 		{
 			if (fields != null)
 				return fields;
 
 			int field_count = MonoRuntime.MonoClassGetFieldCount (target, KlassAddress);
+			if ((field_count != 0) && !MonoRuntime.MonoClassHasFields (target, KlassAddress))
+				throw new TargetException (TargetError.ClassNotInitialized);
 
 			fields = new MonoFieldInfo [field_count];
 			field_offsets = new int [field_count];
@@ -205,6 +212,9 @@ namespace Mono.Debugger.Languages.Mono
 
 		internal TargetObject GetStaticField (Thread thread, TargetFieldInfo field)
 		{
+			if (!thread.CurrentFrame.Language.IsManaged)
+				throw new TargetException (TargetError.InvalidContext);
+
 			TargetAddress data_address = thread.CallMethod (
 				SymbolFile.MonoLanguage.MonoDebuggerInfo.ClassGetStaticFieldData,
 				KlassAddress, 0);
@@ -274,6 +284,9 @@ namespace Mono.Debugger.Languages.Mono
 		internal void SetStaticField (Thread thread, TargetFieldInfo field,
 					      TargetObject obj)
 		{
+			if (!thread.CurrentFrame.Language.IsManaged)
+				throw new TargetException (TargetError.InvalidContext);
+
 			TargetAddress data_address = thread.CallMethod (
 				SymbolFile.MonoLanguage.MonoDebuggerInfo.ClassGetStaticFieldData,
 				KlassAddress, 0);
