@@ -348,6 +348,10 @@ namespace Mono.Debugger.Languages.Mono
 				});
 		}
 
+		public override bool IsClassInitialized {
+			get { return class_info != null; }
+		}
+
 		MonoClassInfo IMonoStructType.ClassInfo {
 			get { return class_info; }
 			set { class_info = value; }
