@@ -998,20 +998,32 @@ namespace Mono.Debugger.Languages.Mono
 				method_load_handlers.Add (index, handler);
 		}
 
-		internal void RemoveMethodLoadHandler (Thread target, int index)
+		internal void RemoveMethodLoadHandler (Thread thread, int index)
+		{
+			if (!thread.CurrentFrame.Language.IsManaged)
+				throw new TargetException (TargetError.InvalidContext);
+
+			thread.CallMethod (info.RemoveBreakpoint, index, 0);
+			method_load_handlers.Remove (index);
+		}
+
+		internal void RemoveMethodLoadHandler (int index)
 		{
-			target.CallMethod (info.RemoveBreakpoint, index, 0);
 			method_load_handlers.Remove (index);
 		}
 
 		internal int RegisterMethodLoadHandler (Thread thread, MonoFunctionType func,
 							FunctionBreakpointHandle handle)
 		{
-			int index = GetUniqueID ();
+			if (method_load_handlers.Contains (handle.Index))
+				return handle.Index;
+
+			if (!thread.CurrentFrame.Language.IsManaged)
+				throw new TargetException (TargetError.InvalidContext);
 
 			TargetAddress retval = thread.CallMethod (
 				info.InsertSourceBreakpoint, func.SymbolFile.MonoImage,
-				func.Token, index, func.DeclaringType.BaseName);
+				func.Token, handle.Index, func.DeclaringType.BaseName);
 
 			MethodLoadedHandler handler = handle.MethodLoaded;
 
@@ -1024,8 +1036,8 @@ namespace Mono.Debugger.Languages.Mono
 				});
 			}
 
-			method_load_handlers.Add (index, handler);
-			return index;
+			method_load_handlers.Add (handle.Index, handler);
+			return handle.Index;
 		}
 #endregion
 
@@ -1189,6 +1201,9 @@ namespace Mono.Debugger.Languages.Mono
 			} else
 				throw new InvalidOperationException ();
 
+			if (!thread.CurrentFrame.Language.IsManaged)
+				throw new TargetException (TargetError.InvalidContext);
+
 			TargetAddress boxed = thread.CallMethod (MonoDebuggerInfo.GetBoxedObjectMethod, klass, value);
 			if (boxed.IsNull)
 				return null;
@@ -1385,6 +1400,8 @@ namespace Mono.Debugger.Languages.Mono
 				MonoSymbolFile symfile = load_symfile (inferior, data);
 				Report.Debug (DebugFlags.JitSymtab,
 					      "Module load: {0} {1}", data, symfile);
+				if (symfile == null)
+					break;
 				engine.Process.Debugger.OnModuleLoaded (symfile.Module);
 				if ((builtin_types != null) && (symfile != null)) {
 					if (engine.OnModuleLoaded (symfile.Module))
