@@ -79,6 +79,24 @@ namespace Mono.Debugger.Languages.Mono
 			return memory.ReadInteger (klass + MonoMetadataInfo.KlassTokenOffset);
 		}
 
+		public int MonoClassGetInstanceSize (TargetMemoryAccess memory,
+						     TargetAddress klass)
+		{
+			int flags = memory.ReadInteger (klass + 4 * memory.TargetAddressSize);
+
+			bool size_inited = (flags & 4) != 0;
+			bool valuetype = (flags & 8) != 0;
+
+			if (!size_inited)
+				throw new TargetException (TargetError.ClassNotInitialized);
+
+			int size = memory.ReadInteger (klass + 4 + 3 * memory.TargetAddressSize);
+			if (valuetype)
+				size -= 2 * memory.TargetAddressSize;
+
+			return size;
+		}
+
 		public TargetAddress MonoClassGetParent (TargetMemoryAccess memory,
 							 TargetAddress klass)
 		{
@@ -103,6 +121,13 @@ namespace Mono.Debugger.Languages.Mono
 			return klass + MonoMetadataInfo.KlassByValArgOffset;
 		}
 
+		public bool MonoClassHasFields (TargetMemoryAccess memory, TargetAddress klass)
+		{
+			TargetAddress fields = memory.ReadAddress (
+				klass + MonoMetadataInfo.KlassFieldOffset);
+			return !fields.IsNull;
+		}
+
 		public int MonoClassGetFieldCount (TargetMemoryAccess memory, TargetAddress klass)
 		{
 			return memory.ReadInteger (klass + MonoMetadataInfo.KlassFieldCountOffset);
@@ -116,6 +141,8 @@ namespace Mono.Debugger.Languages.Mono
 
 			TargetAddress fields = memory.ReadAddress (
 				klass + MonoMetadataInfo.KlassFieldOffset);
+			if (fields.IsNull)
+				throw new TargetException (TargetError.ClassNotInitialized);
 
 			return memory.ReadAddress (fields + offset);
 		}
@@ -128,6 +155,8 @@ namespace Mono.Debugger.Languages.Mono
 
 			TargetAddress fields = memory.ReadAddress (
 				klass + MonoMetadataInfo.KlassFieldOffset);
+			if (fields.IsNull)
+				throw new TargetException (TargetError.ClassNotInitialized);
 
 			return memory.ReadInteger (fields + offset);
 		}
@@ -150,6 +179,9 @@ namespace Mono.Debugger.Languages.Mono
 			TargetAddress methods = memory.ReadAddress (
 				klass + MonoMetadataInfo.KlassMethodsOffset);
 
+			if (methods.IsNull)
+				throw new TargetException (TargetError.ClassNotInitialized);
+
 			methods += index * memory.TargetAddressSize;
 			return memory.ReadAddress (methods);
 		}
@@ -594,7 +626,11 @@ namespace Mono.Debugger.Languages.Mono
 			reader.BinaryReader.ReadInt16 (); /* flags */
 			int pos = reader.BinaryReader.ReadInt16 ();
 
-			string name = memory.ReadString (name_addr);
+			string name;
+			if (!name_addr.IsNull)
+				name = memory.ReadString (name_addr);
+			else
+				name = String.Format ("!{0}", pos);
 
 			return new GenericParamInfo (container, klass, name, pos);
 		}
