@@ -4,12 +4,13 @@ using NUnit.Framework;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
 using Mono.Debugger.Frontend;
+using Mono.Debugger.Test.Framework;
 using EE = Mono.Debugger.ExpressionEvaluator;
 
 namespace Mono.Debugger.Tests
 {
-	[TestFixture]
-	public class TestExpressionEvaluator : TestSuite
+	[DebuggerTestFixture]
+	public class TestExpressionEvaluator : DebuggerTestFixture
 	{
 		public TestExpressionEvaluator ()
 			: base ("TestExpressionEvaluator")
@@ -33,7 +34,7 @@ namespace Mono.Debugger.Tests
 
 			EE.EvaluationFlags flags = EE.EvaluationFlags.NestedBreakStates;
 
-			EE.AsyncResult async = expr.EvaluateAsync (thread.CurrentFrame, flags,
+			EE.AsyncResult async = expr.Evaluate (thread.CurrentFrame, flags,
 				delegate (EE.EvaluationResult result, object data) {
 					Console.WriteLine ("EVALUATION DONE: {0} {1}", result, data);
 			});
